/**
 * @license
 * Copyright (c) 2016 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
import { AnalyzerCacheContext } from '../core/analyzer-cache-context';
import { ParsedDocument } from '../parser/document';
import { Behavior } from '../polymer/behavior';
import { DomModule } from '../polymer/dom-module-scanner';
import { PolymerElement } from '../polymer/polymer-element';
import { Warning } from '../warning/warning';
import { Element } from './element';
import { ElementReference } from './element-reference';
import { Feature, ScannedFeature } from './feature';
import { Import } from './import';
import { Queryable } from './queryable';
import { SourceRange } from './source-range';
/**
 * The metadata for all features and elements defined in one document
 */
export declare class ScannedDocument {
    document: ParsedDocument<any, any>;
    features: ScannedFeature[];
    isInline: boolean;
    sourceRange: SourceRange | undefined;
    warnings: Warning[];
    constructor(document: ParsedDocument<any, any>, features: ScannedFeature[], warnings?: Warning[]);
    readonly url: string;
    /**
     * Gets all features in this scanned document and all inline documents it
     * contains.
     */
    getNestedFeatures(): ScannedFeature[];
    private _getNestedFeatures(features);
}
export interface FeatureKinds {
    'document': Document;
    'element': Element;
    'polymer-element': PolymerElement;
    'behavior': Behavior;
    'dom-module': DomModule;
    'element-reference': ElementReference;
    'import': Import;
    'html-document': Document;
    'js-document': Document;
    'json-document': Document;
    'css-document': Document;
    'html-import': Import;
    'html-script': Import;
    'html-style': Import;
    'js-import': Import;
}
export declare class Document implements Feature, Queryable {
    kinds: Set<string>;
    identifiers: Set<string>;
    analyzer: AnalyzerCacheContext;
    warnings: Warning[];
    private _localFeatures;
    private _scannedDocument;
    /** See parsedDocument. */
    astNode: null;
    /**
     * To handle recursive dependency graphs we must track whether we've started
     * resolving this Document so that we can reliably early exit even if one
     * of our dependencies tries to resolve this document.
     */
    private _begunResolving;
    /**
     * True after this document and all of its children are finished resolving.
     */
    private _doneResolving;
    constructor(base: ScannedDocument, analyzer: AnalyzerCacheContext);
    readonly url: string;
    readonly isInline: boolean;
    readonly parsedDocument: ParsedDocument<any, any>;
    readonly sourceRange: SourceRange | undefined;
    readonly resolved: boolean;
    readonly type: string;
    /**
     * Resolves all features of this document, so that they have references to all
     * their dependencies.
     *
     * This method can only be called once
     */
    resolve(): void;
    /**
     * Adds and indexes a feature to this documentled before resolve().
     */
    _addFeature(feature: Feature): void;
    getByKind<K extends keyof FeatureKinds>(kind: K): Set<FeatureKinds[K]>;
    getByKind(kind: string): Set<Feature>;
    getById<K extends keyof FeatureKinds>(kind: K, identifier: string): Set<FeatureKinds[K]>;
    getById(kind: string, identifier: string): Set<Feature>;
    getOnlyAtId<K extends keyof FeatureKinds>(kind: K, identifier: string): FeatureKinds[K] | undefined;
    getOnlyAtId(kind: string, identifier: string): Feature | undefined;
    private _getByKind(kind, documentsWalked);
    /**
     * Get features for all documents reachable via imports in this document.
     * If `deep` is false, only return features in this document.
     */
    getFeatures(deep?: boolean): Set<Feature>;
    private _getFeatures(result, visited, deep);
    /**
     * Get warnings for this document and all local features of this document. If
     * `deep` is true, return warnings for all documents and features reachable
     * via imports in this document.
     */
    getWarnings(deep?: boolean): Warning[];
    toString(): string;
    private _toString(documentsWalked);
    stringify(): string;
    private _featuresByKind;
    private _featuresByKindAndId;
    private _initIndexes();
    private _indexFeature(feature);
    private _buildIndexes();
}
